//-----------------------------------------------------------------------------
//file : ImageDirectory.cs
//brief : Directory Object
//author : Benjamin Maillard
//date : 31 Mars 2009
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;
using System.IO;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Windows.Input;

namespace TriPhoto
{
    /// <summary>
    /// Repertoire contenant les images
    /// </summary>
    public sealed class ImageDirectory
    {
        #region membres
        private string _name = String.Empty;
        private List<ImageElement> _elements = new List<ImageElement>();
        private string _directoryBase = String.Empty;
        private List<string> _fileList = new List<string>();
        #endregion

        #region event
        public delegate void CopieProgressEvent(string in_text);

        public event CopieProgressEvent CopieProgress;
        #endregion

        /// <summary>
        /// Constructeur
        /// </summary>
        public ImageDirectory()
        { 
            _name = "";
        }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="name"></param>
        public ImageDirectory(string name)
        {
            _name = name;
        }

        #region proprits
        /// <summary>
        /// Nom du repertoire
        /// </summary>
        public string Name
        { 
            get
            {
                return _name;
            }
            set 
            {
                _name = value;
            }
        }
        /// <summary>
        /// Listes des images 
        /// </summary>
        ///         
        [System.Xml.Serialization.XmlIgnoreAttribute]
        public List<ImageElement> ImageElements
        {
            get
            {
                return _elements;
            }
        }

        /// <summary>
        /// Repertoire de base
        /// </summary>
        public string DirectoryBase
        {
            set
            {
                _directoryBase = value;
            }
        }

        /// <summary>
        /// List des fichier image
        /// Attention sert uniquement a la serialisation
        /// Sur le set recharge les Images
        /// </summary>
        public List<string> FileList
        {
            get 
            {
                if(_fileList == null)
                    _fileList = new List<string>();

                if (_elements.Count != 0)
                {
                    foreach (ImageElement l_image in _elements)
                    {
                        _fileList.Add(l_image.Uri.OriginalString);
                    }
                }

                return _fileList;
            }
            set
            {
                _fileList = value;
                foreach (string l_filePath in _fileList)
                {
                    _elements.Add(new ImageElement(l_filePath));
                }
            }
        }

        /// <summary>
        /// Rechergement des images a partir de la liste des fichiers
        /// </summary>
        /// <returns></returns>
        public void ReloadDirectory(MouseButtonEventHandler l_mouseUpCallBack, ImageState in_imageState)
        {
            _elements.Clear();
            foreach (string l_filePath in _fileList)
            {
                ImageElement l_imageElt = new ImageElement(l_filePath);
                l_imageElt.ImageState = in_imageState;
                l_imageElt.Cursor = System.Windows.Input.Cursors.Hand;
                l_imageElt.MouseUp += new MouseButtonEventHandler(l_mouseUpCallBack);

                _elements.Add(l_imageElt);
            }
        }

        /// <summary>
        /// Assigne la largeur max a un element
        /// </summary>
        /// <param name="in_width"></param>
        public void ComputeMaxWidthFromWidth(double in_width)
        {
            foreach (ImageElement l_image in _elements)
                l_image.ComputeMaxWidthFromWidth(in_width);
        }

        /// <summary>
        /// Nombre d'image dans le repertoire
        /// </summary>
        ///         
        [System.Xml.Serialization.XmlIgnoreAttribute]
        public int NbImageInDirectory
        {
            get
            {
                return _elements.Count;
            }
        }
        #endregion

        #region mthodes
        /// <summary>
        /// Ajoute une images dans le repertoire
        /// </summary>
        /// <param name="in_image"></param>
        public void AddImage(ImageElement in_image)
        {
            _elements.Add(in_image);
        }

        /// <summary>
        /// Suppression d'une image d'un repertoire
        /// </summary>
        /// <param name="in_image"></param>
        /// <returns></returns>
        public bool RemImage(ImageElement in_image)
        {
            return _elements.Remove(in_image);
        }

        /// <summary>
        /// Copie les photos dans le repertoire
        /// </summary>
        /// <returns></returns>
        public bool Transfert(bool in_applyRotation)
        {
            string l_directory = string.Format(@"{0}\{1}",_directoryBase,_name);

            try
            {
                //On supprime le repertoire ainsi que toutes les images quil contient
                ReinitDirectory(l_directory);

                //Creation du repertoire si il n'existe pas
                if (Directory.Exists(l_directory) == false && _elements.Count > 0)
                    Directory.CreateDirectory(l_directory);

                foreach (ImageElement l_image in _elements)
                {
                    //On declenche l'evenement
                    if (CopieProgress != null)
                        CopieProgress(string.Format("{0} {1}", TriPhoto.Properties.Resources.CopyImage, l_image.Uri.OriginalString));

                    //Copie de l'image
                    if (l_image.CopieTo(l_directory, in_applyRotation) == false)
                    {
                        //Oups une erreur
                        MessageBox.Show(string.Format("{0} : {1}", 
                            TriPhoto.Properties.Resources.CopyError,
                            l_image.Uri.OriginalString));
                    }
                }
            }
            catch (Exception ex)
            {
                //Oups une erreur grave
                MessageBox.Show(string.Format("{0} : {1}",
                    TriPhoto.Properties.Resources.TransfertError,
                    l_directory));
            }

            return true;
        }

        /// <summary>
        /// Efface toute les images du repertoire
        /// </summary>
        /// <returns></returns>
        private bool ReinitDirectory(string in_directory)
        {
            //Si le reperoire existe on le supprime
            if (Directory.Exists(in_directory) == true)
                Directory.Delete(in_directory, true);

            return true;
        }
        #endregion
    }
}
