//-----------------------------------------------------------------------------
//file : ImageElement.cs
//brief : Image Object
//author : Benjamin Maillard
//date : 31 Mars 2009
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows;
using System.Windows.Media.Effects;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.IO;
using System.Collections.ObjectModel;

namespace TriPhoto
{
    public enum ImageState { AUCUN, NON_CLASSE, A_GARDER, A_SUPPRIMER, NON_DETERMINE }

    /// <summary>
    /// Classe reprsentant une image a trier
    /// </summary>
    public class ImageElement : Image
    {
        #region membres
        private ImageState imageState = ImageState.AUCUN;
        private Border border = null;
        private Label dateLabel = null;
        private Border viewBorder = null;
        private Image viewImage = null;
        private CroppedBitmap crop = null;
        private double rotation = 0;
        private double OX = 0;
        private double OY = 0;
        private bool selected = false;
        private bool viewSelected = false;
        private bool inGray = false;
        private Uri uriSource = null;
        private DateTime datePriseDeVue = DateTime.Now;
        #endregion

        #region proprit
        /// <summary>
        /// Etat de l'image
        /// </summary>
        public ImageState ImageState
        { 
            get
            {
                return imageState;
            }
            set
            {
                if (imageState == value)
                    return;

                ImageElementManager.Manager.MoveImageInState(this, value);

                imageState = value;
            }
        }

        /// <summary>
        /// Image de visualisation en pleine resolution
        /// </summary>
        public Image ViewImage
        {
            get
            {
                if (viewImage == null)
                {
                    viewImage = new Image();

                    BitmapImage l_bitmap = new BitmapImage();
                    l_bitmap.BeginInit();
                    l_bitmap.UriSource = uriSource;
                    l_bitmap.EndInit();

                    CachedBitmap l_cache = new CachedBitmap(l_bitmap, BitmapCreateOptions.None, BitmapCacheOption.OnLoad);
                    
                    Int32Rect l_rect = new Int32Rect(0, 0, l_bitmap.PixelWidth, l_bitmap.PixelHeight);
                    crop = new CroppedBitmap(l_cache, l_rect);

                    BitmapSource l_rotation = new TransformedBitmap(crop, new RotateTransform(rotation));

                    if (inGray == true)
                    {
                        l_rotation = new FormatConvertedBitmap(l_rotation, PixelFormats.Gray8, BitmapPalettes.Gray256, 1.0);
                    }

                    viewImage.Source = l_rotation;
                    viewImage.Margin = new Thickness(10);
                    viewImage.Tag = this;
                    viewImage.MouseDown += new System.Windows.Input.MouseButtonEventHandler(viewImage_MouseDown);

                    viewBorder = new Border();
                    viewBorder.Child = viewImage;
                    viewBorder.BorderBrush = Brushes.Gray;
                    viewBorder.Padding = new Thickness(0);
                    viewBorder.Margin = new Thickness(4);
                    viewBorder.BorderThickness = new Thickness(2);
                    viewBorder.CornerRadius = new CornerRadius(4);
                    viewBorder.Tag = this;
                }

                
                return viewImage;
            }
            set
            {
                viewBorder.MouseEnter -= new System.Windows.Input.MouseEventHandler(viewBorder_MouseEnter);
                viewBorder.MouseLeave -= new System.Windows.Input.MouseEventHandler(viewBorder_MouseLeave);

                viewBorder = null;
                viewImage = null;
                crop = null;
            }
        }

        /// <summary>
        /// Sur click dans l'image de visualisation on prepare le drag and Drop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void viewImage_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            List<ImageElement> l_list = new List<ImageElement>();
            l_list.Add(this);
            DataObject l_data = new DataObject(DataFormats.Locale,l_list);
            
            DragDropEffects l_effect = DragDrop.DoDragDrop(this, l_data, DragDropEffects.Move);
        }

        /// <summary>
        /// Sur click dans l'image on prepare le drag and Drop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ImageElement_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (e.LeftButton == System.Windows.Input.MouseButtonState.Released)
                return;

            List<ImageElement> l_list = ImageElementManager.Manager.GetSelectedImagette();
            l_list.Add(this);
            DataObject l_data = new DataObject(DataFormats.Locale, l_list);

            DragDropEffects l_effect = DragDrop.DoDragDrop(this, l_data, DragDropEffects.Move);
        }

      
        /// <summary>
        /// Bordure de l'imagette
        /// </summary>
        public Border Border
        {
            get
            {
                if (border == null)
                {
                    border = new Border();

                    StackPanel l_panel = new StackPanel();

                    dateLabel = new Label();
                    dateLabel.Content = DatePriseDeVue.ToString();
                    l_panel.Children.Add(dateLabel);
                    l_panel.Children.Add(this);

                    border.Child = l_panel;
                    border.Tag = this;

                    border.MouseEnter += new System.Windows.Input.MouseEventHandler(border_MouseEnter);
                    border.MouseLeave += new System.Windows.Input.MouseEventHandler(border_MouseLeave);
                }

                return border;
            }
            
            set
            {
                viewBorder = null;
                viewImage = null;
            }
        }

        /// <summary>
        /// Acces au exif
        /// </summary>
        public ImageExifMetadata ImageExifMetadata
        {
            get {
                return new ImageExifMetadata(uriSource);
            }
        }


        public Uri Uri
        {
            get
            {
                return uriSource;
            }
        }

        /// <summary>
        /// Date de prise de vue de la photo
        /// </summary>
        public DateTime DatePriseDeVue
        {
            get
            {
                return datePriseDeVue;
            }
        }

        /// <summary>
        /// Calcul des bordures de l'image
        /// </summary>
        /// <param name="in_maxWidth"></param>
        public void ComputeMaxWidthFromWidth(double in_maxWidth)
        {
            /*if (this.Source.Height > this.Source.Width)
            {*/
                double l_factor = this.Source.Width / this.Source.Height;
                border.MaxWidth = in_maxWidth * l_factor;
           /* }
            else
            {
                double l_factor = this.Source.Height / this.Source.Width;
                border.MaxHeight = in_maxWidth * l_factor;
            }*/
        }

        /// <summary>
        /// Bordure de l'image visualis
        /// </summary>
        public Border ViewImageBorder
        {
            get
            {
                if (viewBorder == null)
                {
                    Image l_bidonImage = ViewImage;
                }

                return viewBorder;
            }
        }

        /// <summary>
        ///  Animation a la sortie de la bordure
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void border_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            ThicknessAnimation l_thicknessAnimation =
                        new ThicknessAnimation(
                        new Thickness(4),
                        new Duration(TimeSpan.FromSeconds(0.2)));

            AnimationClock l_clock = l_thicknessAnimation.CreateClock();
            border.ApplyAnimationClock(System.Windows.Controls.Border.MarginProperty, l_clock);
        }

        /// <summary>
        /// Animation a l'entre sur la bordure
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void border_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            ThicknessAnimation l_thicknessAnimation =
                        new ThicknessAnimation(
                        new Thickness(0),
                        new Duration(TimeSpan.FromSeconds(0.2)));

            AnimationClock l_clock = l_thicknessAnimation.CreateClock();
            border.ApplyAnimationClock(System.Windows.Controls.Border.MarginProperty, l_clock);
        }

        /// <summary>
        ///  Animation a la sortie de la visu de l'image
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void viewBorder_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            ThicknessAnimation l_thicknessAnimation =
                        new ThicknessAnimation(
                        new Thickness(4),
                        new Duration(TimeSpan.FromSeconds(0.2)));

            AnimationClock l_clock = l_thicknessAnimation.CreateClock();
            viewBorder.ApplyAnimationClock(System.Windows.Controls.Border.MarginProperty, l_clock);
        }

        /// <summary>
        ///  Animation a l'entr de la visu de l'image
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void viewBorder_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            ThicknessAnimation l_thicknessAnimation =
                        new ThicknessAnimation(
                        new Thickness(0),
                        new Duration(TimeSpan.FromSeconds(0.2)));

            AnimationClock l_clock = l_thicknessAnimation.CreateClock();
            viewBorder.ApplyAnimationClock(System.Windows.Controls.Border.MarginProperty, l_clock);
        }

        /// <summary>
        /// Selection de l'imagette
        /// </summary>
        public bool Select
        {
            get
            {
                return selected;
            }
            set
            {
                if (value != selected)
                {
                    selected = value;
                    if (selected)
                    {
                        border.Background = Brushes.YellowGreen;
                    }
                    else 
                    {
                        border.Background = Brushes.White;
                    }
                }
            }
        }

        /// <summary>
        /// Selection de l'image de visualisation
        /// </summary>
        public bool SelectViewImage
        {
            get
            {
                return viewSelected;
            }
            set
            {
                if (value != viewSelected)
                {
                    viewSelected = value;
                    if (viewSelected)
                    {
                        viewBorder.BorderBrush = Brushes.Yellow;
                    }
                    else 
                    {
                        viewBorder.BorderBrush = Brushes.Gray;
                    }
                }
            }
        }
        #endregion

        #region constructeur
        public ImageElement(string in_path)
        {
            Charger(in_path);

            Border.Background = Brushes.White;
            Border.Padding = new Thickness(6);
            Border.Margin = new Thickness(4);

            this.MouseLeave += new System.Windows.Input.MouseEventHandler(ImageElement_MouseLeave);
        }


        #endregion

        #region mthodes
        /// <summary>
        /// Charge l'image en parametre
        /// </summary>
        /// <param name="in_imagePath"></param>
        public void Charger(string in_imagePath)
        {
            uriSource = new Uri(in_imagePath, UriKind.Absolute);

            BitmapFrame l_bitmapFrame = BitmapFrame.Create(uriSource,
                                                           BitmapCreateOptions.DelayCreation,
                                                           BitmapCacheOption.OnDemand);

            ImageExifMetadata imageExifMetadata = null;

            //Si un thumtail existe
            if (l_bitmapFrame.Thumbnail != null)
            {
                Source = l_bitmapFrame.Thumbnail;

                //Lecture des metadonnes
                imageExifMetadata = new ImageExifMetadata(l_bitmapFrame);
            }
            else
            {
                //Sinon methode traditionel
                BitmapImage l_bitmap = new BitmapImage();
                l_bitmap.BeginInit();
                l_bitmap.UriSource = uriSource;
                l_bitmap.DecodePixelWidth = 180;
                l_bitmap.EndInit();
            
                Source = l_bitmap;

                //Lecture des metadonnes
                imageExifMetadata = new ImageExifMetadata(uriSource);
            }

            if(imageExifMetadata.DateImageTaken != null)
                datePriseDeVue = imageExifMetadata.DateImageTaken.Value;
        }

        /// <summary>
        /// Transforme l'imge visualise en noir et blanc
        /// </summary>
        public void ToBlackAndWhite()
        {
            if (inGray == false)
            {
                BitmapSource l_source = (BitmapSource)(ViewImage.Source);
                ViewImage.Source = new FormatConvertedBitmap(l_source, PixelFormats.Gray8, BitmapPalettes.Gray256, 1.0);
                inGray = true;
            }
        }

        /// <summary>
        /// Transforme l'image en couleur
        /// </summary>
        public void ToColor()
        {
            if (inGray == true)
            {
                FormatConvertedBitmap l_image = ViewImage.Source as FormatConvertedBitmap;
                ViewImage.Source = l_image.Source;
                inGray = false;
            }
        }

        /// <summary>
        /// Passe l'image alternativement de couleur a gris
        /// </summary>
        public void ChangeColorMode()
        {
            if (inGray)
                ToColor();
            else
                ToBlackAndWhite();
        }

        /// <summary>
        /// Rotation de 90 de l'image
        /// </summary>
        public void RotateClockWise()
        {
            rotation = (rotation - 90) % 360;
            Rotate();
        }

        /// <summary>
        /// Rotation de 90 de l'image
        /// </summary>
        public void RotateUnClockWise()
        {
            rotation = (rotation + 90) % 360;
            Rotate();
        }

        /// <summary>
        /// Applique la rotation a l'image
        /// </summary>
        private void Rotate()
        {
            BitmapSource l_rotation = new TransformedBitmap(crop, new RotateTransform(rotation));

            if (inGray == true)
            {
                l_rotation = new FormatConvertedBitmap(l_rotation, PixelFormats.Gray8, BitmapPalettes.Gray256, 1.0);
            }

            viewImage.Source = l_rotation;
        }

        /// <summary>
        /// Copie l'image dans le reperoire
        /// </summary>
        /// <param name="in_path"></param>
        /// <returns></returns>
        public bool CopieTo(string in_path, bool in_applyRotation)
        {
            string l_destination = String.Format(@"{0}\{1}", in_path, Path.GetFileName(uriSource.AbsolutePath));

            //Si pas de transformation et pas de rotation je copie l'image
            if (in_applyRotation == false || (rotation == 0 && inGray == false))
            {

                try
                {
                    if (File.Exists(l_destination) == false)
                        File.Copy(uriSource.OriginalString, l_destination);
                }
                catch (Exception ex)
                {
                    return false;
                }
            }
            else //Sinon on applique les transformations et on sauvegarde l'image
            {
                try
                {
                    //On supprime le fichier si il existe
                    if (File.Exists(l_destination) == true)
                    {
                        File.Delete(l_destination);
                    }

                    //Lecture de l'image et de ses metadonnes
                    BitmapDecoder l_BitmapDeco = BitmapDecoder.Create(
                                new Uri(uriSource.OriginalString, UriKind.Absolute),
                                BitmapCreateOptions.None,
                                BitmapCacheOption.Default);


                    //Rotation de l'image
                    BitmapSource l_rotation = new TransformedBitmap(l_BitmapDeco.Frames[0] as BitmapSource, new RotateTransform(rotation));
                    ReadOnlyCollection<ColorContext> l_colorContext = new ReadOnlyCollection<ColorContext> (l_BitmapDeco.Frames[0].ColorContexts);

                    //En noir et blanc
                    if (inGray == true)
                    {
                        l_rotation = new FormatConvertedBitmap(l_rotation, PixelFormats.Gray8, BitmapPalettes.Gray256, 1.0);
                    }
                    
                    //On encode en JPEG
                    JpegBitmapEncoder l_jpegEncoder = new JpegBitmapEncoder();
                    using (FileStream l_stream = new FileStream(l_destination, FileMode.Create))
                    {

                        l_jpegEncoder.FlipHorizontal = false;
                        l_jpegEncoder.FlipVertical = false;
                        l_jpegEncoder.QualityLevel = 100;
                        l_jpegEncoder.Rotation = Rotation.Rotate0;
                        //Create Thumbnail
                        BitmapSource l_thumbs = new TransformedBitmap(l_rotation, new ScaleTransform(0.1, 0.1));

                        BitmapFrame l_bitmapFrame = BitmapFrame.Create(l_rotation, l_thumbs, l_BitmapDeco.Frames[0].Metadata.Clone() as BitmapMetadata, l_colorContext);

                        l_jpegEncoder.Frames.Add(l_bitmapFrame);

                        l_jpegEncoder.Save(l_stream);
                    }
                    
                    //Liberation de la memoire
                    l_rotation = null;
                    l_jpegEncoder = null;
                }
                catch (Exception ex)
                {
                    return false;
                }
            }

            return true;
        }

        #endregion
    }
}
