//-----------------------------------------------------------------------------
//file : App.xaml.cs
//brief : Image Object Manager
//author : Benjamin Maillard
//date : 31 Mars 2009
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;
using System.Windows.Media.Effects;
using System.Windows.Media.Animation;
using System.Windows;
using System.Windows.Input;
using System.IO;
using System.Windows.Media.Imaging;
using System.Windows.Media;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml.Serialization;

namespace TriPhoto
{
    /// <summary>
    /// Classe de gestion des images (etat, liste, ...)
    /// </summary>
    sealed class  ImageElementManager
    {
        #region membres
        private Panel listImageNonClasser;
        private Panel listImageASupprimer;
        private Panel listImageADeterminer;
        private Panel listImageAGarder;
        private Grid principaleView;
        private Canvas popupView;
        private bool compareMode;
        private bool rowOrColunm = false;
        private string directoryBase;

        private static string[] listFileStringExtension = { "*.jpg", "*.tif", "*.png", "*.gif", "*.wdp" };

        private List<ImageDirectory> directoryList = new List<ImageDirectory>();
        private ImageDirectory imageAGarderDirectory;
        private ImageDirectory imageADeterminerDirectory;
        private ImageDirectory imageASupprimerDirectory;
        private ImageDirectory imageNonClasseDirectory;

        private int _nbImageInDirectories = 0;
        private int _currentCopyProgress = 0;

        private string A_GARDER = TriPhoto.Properties.Resources.IKeepTooltip;
        private string GARDER = TriPhoto.Properties.Resources.Keep;
        private string A_DETERMINER = TriPhoto.Properties.Resources.ToDetermine;
        private string A_SUPPRIMER = TriPhoto.Properties.Resources.ToDelete;
        private string NON_CLASSE = TriPhoto.Properties.Resources.NotClassified;

        private const int MAX_WIDTH = 100;

        public static readonly ImageElementManager Manager = new ImageElementManager();
        #endregion

        #region proprits
        public Panel ListImageNonClasser
        {
            set { listImageNonClasser = value; }
        }
        public Panel ListImageASupprimer
        {
            set { listImageASupprimer = value; }
        }
        public Panel ListImageADeterminer
        {
            set { listImageADeterminer = value; }
        }
        public Panel ListImageAGarder
        {
            set { listImageAGarder = value; }
        }
        public Grid PrincipaleView
        {
            set { principaleView = value; }
        }
        public Canvas PopupView
        {
            get 
            {
                return popupView;
            }
            set
            {
                popupView = value;
            }
        }
        public bool CompareMode
        {
            set
            {
                compareMode = value;
                if (compareMode == false)
                    Unshow();              
            }
            get
            {
                return compareMode;
            }
        }
        #endregion

        /// <summary>
        /// Constructeur
        /// </summary>
        public ImageElementManager()
        {
            imageAGarderDirectory = new ImageDirectory(GARDER);
            directoryList.Add(imageAGarderDirectory);

            imageADeterminerDirectory = new ImageDirectory(A_DETERMINER);
            imageASupprimerDirectory = new ImageDirectory(A_SUPPRIMER);
            imageNonClasseDirectory = new ImageDirectory(NON_CLASSE);
        }

        #region evenement
        public delegate void LoadBeginEvent();
        public delegate void LoadEndEvent();
        public delegate void LoadProgressEvent(int in_pourcent, string in_text);
        /// <summary>
        /// Appell lors du chargement d'un repertoire et de la copie des images
        /// </summary>
        public event LoadProgressEvent LoadProgress = null;
        /// <summary>
        /// Appel lors du debut du chargement et de copie
        /// </summary>
        public event LoadBeginEvent LoadBegin = null;
        /// <summary>
        /// Appel en fin de chargement et de copie
        /// </summary>
        public event LoadEndEvent LoadEnd = null;
        #endregion

        #region mthodes

        #region Gestion des repertoires
        /// <summary>
        /// Ajout d'un repertoire
        /// </summary>
        /// <param name="in_name"></param>
        public void AddDirectory(string in_name)
        {
            ImageDirectory l_directory = new ImageDirectory(in_name);

            l_directory.DirectoryBase = directoryBase;
            directoryList.Add(l_directory);
            imageAGarderDirectory = l_directory;
        }

        /// <summary>
        /// Suppression d'un repertoire
        /// </summary>
        /// <param name="Name"></param>
        public bool RemDirectoy(string in_name)
        {
            //REcherche du repertoire
            ImageDirectory l_suppDirectory = FindDirectory(in_name);

            //Si pas trouv
            if (l_suppDirectory == null)
                return false;

            //On fait une copie de la liste
            List<ImageElement> l_copyList = new List<ImageElement>(l_suppDirectory.ImageElements);

            //On remet les images du repertoire dans l'etat Non Classe
            foreach (ImageElement l_image in l_copyList)
            {
                l_image.ImageState = ImageState.NON_CLASSE;
            }

            //On vide la liste des images
            l_suppDirectory.ImageElements.Clear();

            //Suppression du repertoire
            directoryList.Remove(l_suppDirectory);

            return true;
        }

        /// <summary>
        /// Renommer le repertoire
        /// </summary>
        /// <param name="in_name"></param>
        /// <param name="in_newName"></param>
        /// <returns></returns>
        public bool RenomerDirectory(string in_name, string in_newName)
        {
            //REcherche du repertoire
            ImageDirectory l_renDirectory = FindDirectory(in_name);

            //Si pas trouv
            if (l_renDirectory == null)
                return false;

            l_renDirectory.Name = in_newName;

            return true;
        }

        /// <summary>
        /// Changement de repertoire de transfert courrant
        /// </summary>
        /// <param name="in_name">nom du repertoire</param>
        public bool SetDefaultDirectory(string in_name)
        { 
            //Recherche du repertoire
            ImageDirectory l_directory = FindDirectory(in_name);
            if(l_directory == null)
                return false;

            //Reperoire courrant
            imageAGarderDirectory = l_directory;

            //On vide les images a garder
            listImageAGarder.Children.Clear();

            //On recharge les images du repertoire
            foreach (ImageElement l_image in l_directory.ImageElements)
            {
                listImageAGarder.Children.Add(l_image.Border);
            }
            
            return true;
        }

        /// <summary>
        /// Recherche un repertoire a partir de son nom
        /// </summary>
        /// <param name="in_name"></param>
        /// <returns></returns>
        private ImageDirectory FindDirectory(string in_name)
        {
            ImageDirectory l_findDirectory = null;

            foreach (ImageDirectory l_directory in directoryList)
            {
                if (l_directory.Name == in_name)
                {
                    l_findDirectory = l_directory;
                    break;
                }
            }
            
            return l_findDirectory;
        }
        #endregion

        #region Sauvegarde et rechargement
        /// <summary>
        /// Sauvegarde le projet dans un fichier
        /// </summary>
        /// <param name="in_filePath"></param>
        /// <returns></returns>
        public bool SaveProject(string in_filePath)
        {

            XmlSerializer l_serializer = new XmlSerializer(typeof(List<ImageDirectory>));
            using (TextWriter l_writer = new StreamWriter(in_filePath))
            {
                List<ImageDirectory> l_listDirectory = new List<ImageDirectory>();
                //l_listDirectory.Add(imageAGarderDirectory);
                l_listDirectory.Add(imageADeterminerDirectory);
                l_listDirectory.Add(imageASupprimerDirectory);
                l_listDirectory.Add(imageNonClasseDirectory);
                l_listDirectory.AddRange(directoryList);

                l_serializer.Serialize(l_writer, l_listDirectory);
                l_writer.Close();
            }

            return true;
        }

        /// <summary>
        /// Chargement du projet
        /// </summary>
        /// <param name="in_filePath"></param>
        /// <returns></returns>
        public bool LoadProject(string in_filePath, ref List<String> in_newDirectory)
        {
            double l_nbImage = 0;
            double l_loadCurrent = 0;
            int l_loadProgress = 0;

            //On vide tout
            Clear();

            //Debut de chargement
            if (LoadBegin != null)
                LoadBegin();

            //Deserialisation du projet
            XmlSerializer l_serializer = new XmlSerializer(typeof(List<ImageDirectory>));
            using (TextReader l_reader = new StreamReader(in_filePath))
            {
                if (LoadProgress != null)
                    LoadProgress(l_loadProgress, "Chargement du fichier projet");

                List<ImageDirectory> l_listDirectory = l_serializer.Deserialize(l_reader)as List<ImageDirectory>;

                directoryList.Clear();
                l_nbImage = l_listDirectory.Count;

                foreach (ImageDirectory l_imageDirectory in l_listDirectory)
                {
                    l_loadProgress = Convert.ToInt32(Math.Floor(l_loadCurrent++ / l_nbImage * 100.0));
                    if (LoadProgress != null)
                        LoadProgress(l_loadProgress, string.Format("Chargement du repertoire {0}",l_imageDirectory.Name));

                    //Classement du repertoire
                    if (l_imageDirectory.Name == A_GARDER)
                    {
                        l_imageDirectory.ReloadDirectory(ImageMouseUpCallBack, ImageState.A_GARDER);
                        imageAGarderDirectory = l_imageDirectory;
                        imageAGarderDirectory.ComputeMaxWidthFromWidth(MAX_WIDTH);
                        directoryList.Add(imageAGarderDirectory);
                    }
                    else if (l_imageDirectory.Name == A_SUPPRIMER)
                    {
                        l_imageDirectory.ReloadDirectory(ImageMouseUpCallBack, ImageState.A_SUPPRIMER);
                        imageASupprimerDirectory = l_imageDirectory;
                        imageASupprimerDirectory.ComputeMaxWidthFromWidth(MAX_WIDTH);
                    }
                    else if (l_imageDirectory.Name == A_DETERMINER)
                    {
                        l_imageDirectory.ReloadDirectory(ImageMouseUpCallBack, ImageState.NON_DETERMINE);
                        imageADeterminerDirectory = l_imageDirectory;
                        imageADeterminerDirectory.ComputeMaxWidthFromWidth(MAX_WIDTH);
                    }
                    else if (l_imageDirectory.Name == NON_CLASSE)
                    {
                        l_imageDirectory.ReloadDirectory(ImageMouseUpCallBack, ImageState.NON_CLASSE);
                        imageNonClasseDirectory = l_imageDirectory;
                        imageNonClasseDirectory.ComputeMaxWidthFromWidth(listImageNonClasser.Width);
                    }
                    else
                    {
                        l_imageDirectory.ReloadDirectory(ImageMouseUpCallBack, ImageState.A_GARDER);
                        directoryList.Add(l_imageDirectory);

                        if (l_imageDirectory.Name != A_GARDER & l_imageDirectory.Name != GARDER)
                            in_newDirectory.Add(l_imageDirectory.Name);

                        l_imageDirectory.ComputeMaxWidthFromWidth(MAX_WIDTH);
                    }
                }
            }

 
            //Fin de chargement
            if (LoadEnd != null)
                LoadEnd();

            return true;
        }

        #endregion

        /// <summary>
        /// Recherche si une case est vide dans la grille
        /// </summary>
        /// <param name="out_row"></param>
        /// <param name="out_colunm"></param>
        /// <returns></returns>
        private bool GetFreeGridStore(out int out_row, out int out_colunm)
        {
            int l_nbColumns = principaleView.ColumnDefinitions.Count;
            int l_nbRows = principaleView.RowDefinitions.Count;

            //Pour toutes les lignes
            for (int l_rowCounter = 0; l_rowCounter < l_nbRows; l_rowCounter++)
            {
                //Pour toutes les colonnes
                for (int l_columnCounter = 0; l_columnCounter < l_nbColumns; l_columnCounter++)
                {

                    bool l_found = false;
                    //On boucle sur toutes les elements pour trouv si un elt est dans la
                    //cellule courrante
                    foreach (UIElement l_elt in principaleView.Children)
                    {
                        int l_currentColumn = Grid.GetColumn(l_elt);
                        int l_currentRow = Grid.GetRow(l_elt);
                        
                        //Trouv ou non
                        if (l_currentColumn == l_columnCounter && l_rowCounter == l_currentRow)
                            l_found = true;
                    }

                    //On a pas trouv d'element donc la cellule est vide
                    if (l_found == false)
                    {
                        out_row = l_rowCounter;
                        out_colunm = l_columnCounter;

                        return true;
                    }
                }
            }

            out_row = -1;
            out_colunm = -1;
            //Pas de cellule vide
            return false;
        }

        /// <summary>
        /// Cr ou retourne l'index d'une cellule libre de la grille
        /// </summary>
        /// <param name="out_row"></param>
        /// <param name="out_colunm"></param>
        private void GetNewGridStore(out int out_row, out int out_colunm)
        {
            //Recherche si une cellule est vide
            if (GetFreeGridStore(out out_row,out out_colunm) == false)
            {
                //Sinon il faut la creer
                //On cre alternativement une colonne une ligne
                if (rowOrColunm == false)
                {
                    principaleView.ColumnDefinitions.Add(new ColumnDefinition());
                    out_row = 0;
                    out_colunm = principaleView.ColumnDefinitions.Count - 1;

                    rowOrColunm = true;
                }
                else
                {
                    principaleView.RowDefinitions.Add(new RowDefinition());
                    out_row = principaleView.RowDefinitions.Count - 1;
                    out_colunm = 0;

                    rowOrColunm = false;
                }
            }
        }

        /// <summary>
        /// Recherche de l'index d'insertion en fonction de la date
        /// </summary>
        /// <param name="in_image"></param>
        /// <returns></returns>
        private int GetInsertIndex(ImageElement in_image)
        {
            int index = 0;
            foreach (Border l_border in listImageNonClasser.Children)
            {
                ImageElement l_image = (ImageElement)l_border.Tag;

                if (l_image.DatePriseDeVue > in_image.DatePriseDeVue)
                    return index;

                index++;
            }

            return index;
        }

        /// <summary>
        /// Change une image d'etat
        /// </summary>
        /// <param name="in_image"></param>
        /// <param name="in_state"></param>
        public void MoveImageInState(ImageElement in_image, ImageState in_state)
        {
            switch (in_image.ImageState)
            { 
                case ImageState.A_SUPPRIMER :
                    listImageASupprimer.Children.Remove(in_image.Border);
                    imageASupprimerDirectory.RemImage(in_image);
                    break;
                case ImageState.NON_CLASSE :
                    listImageNonClasser.Children.Remove(in_image.Border);
                    imageNonClasseDirectory.RemImage(in_image);
                    break;
                case ImageState.NON_DETERMINE :
                    listImageADeterminer.Children.Remove(in_image.Border);
                    imageADeterminerDirectory.RemImage(in_image);
                    break;
                case ImageState.A_GARDER :
                    listImageAGarder.Children.Remove(in_image.Border);
                    imageAGarderDirectory.RemImage(in_image);
                    break;
            }

            switch (in_state)
            {
                case ImageState.A_SUPPRIMER:
                    in_image.ComputeMaxWidthFromWidth(MAX_WIDTH);
                    listImageASupprimer.Children.Insert(0,in_image.Border);
                    imageASupprimerDirectory.AddImage(in_image);
                    break;
                case ImageState.NON_CLASSE:
                    in_image.ComputeMaxWidthFromWidth(listImageNonClasser.Width);
                    //listImageNonClasser.Children.Add(in_image.Border);
                    listImageNonClasser.Children.Insert(GetInsertIndex(in_image), in_image.Border);
                    imageNonClasseDirectory.AddImage(in_image);
                    break;
                case ImageState.NON_DETERMINE:
                    in_image.ComputeMaxWidthFromWidth(MAX_WIDTH);
                    listImageADeterminer.Children.Insert(0,in_image.Border);
                    imageADeterminerDirectory.AddImage(in_image);
                    break;
                case ImageState.A_GARDER:
                    in_image.ComputeMaxWidthFromWidth(MAX_WIDTH);
                    listImageAGarder.Children.Insert(0,in_image.Border);
                    imageAGarderDirectory.AddImage(in_image);
                    break;
            }
        }

        /// <summary>
        /// Affiche une image
        /// </summary>
        /// <param name="in_image"></param>
        public void Show(ImageElement in_image)
        {
            //Je suis en mode une image a la fois
            if (CompareMode == false)
            {
                //L'image est t'elle dejas visualis
                if (principaleView.Children.Contains(in_image.ViewImageBorder))
                    return;

                //Si une image a deja t affich
                if (principaleView.Children.Count > 0)
                {
                    ((principaleView.Children[0] as Border).Tag as ImageElement).Select = false;
                    ((principaleView.Children[0] as Border).Tag as ImageElement).ViewImage = null;
                    principaleView.Children.RemoveAt(0);
                    principaleView.Children.Insert(0, in_image.ViewImageBorder);
                }
                else //Aucune image n'a t affich
                {
                    principaleView.Children.Add(in_image.ViewImageBorder);
                }

                in_image.ViewImage.MouseLeftButtonDown += new MouseButtonEventHandler(ViewImage_MouseLeftButtonDown);

                //On affecte l'image en 0,0
                Grid.SetColumn(in_image.ViewImageBorder, 0);
                Grid.SetRow(in_image.ViewImageBorder, 0);

            }
            else //Je visualise toutes les images que je selectionne
            {
                int l_row = 0, l_colunm = 0;

                //L'image n'est pas visualis il faut l'ajouter a la grille
                if (principaleView.Children.Contains(in_image.ViewImageBorder) == false)
                {
                    GetNewGridStore(out l_row, out l_colunm);
                    principaleView.Children.Add(in_image.ViewImageBorder);

                    in_image.ViewImage.MouseLeftButtonDown += new MouseButtonEventHandler(ViewImage_MouseLeftButtonDown);

                    Grid.SetColumn(in_image.ViewImageBorder, l_colunm);
                    Grid.SetRow(in_image.ViewImageBorder, l_row);
                }
                else //L'image est dans la grille il faut la supprimer
                {
                    principaleView.Children.Remove(in_image.ViewImageBorder);
                    in_image.ViewImage = null;

                }

            }
        }

        /// <summary>
        /// Retourne la liste des images selectionn
        /// </summary>
        /// <returns></returns>
        public List<ImageElement> GetSelectedImage()
        {
            List<ImageElement> l_ImageSelectedList = new List<ImageElement>();

            //Si il n'y a qu'un element alors je consdre qu'il est selectionn dans tous les cas
            if (principaleView.Children.Count == 1)
            {
                l_ImageSelectedList.Add((principaleView.Children[0] as Border).Tag as ImageElement);
            }
            else
            { //Sinon on parcourt les elements
                foreach (Border l_childBorder in principaleView.Children)
                {
                    ImageElement l_imageElement = (l_childBorder.Child as Image).Tag as ImageElement;
                    if (l_imageElement.SelectViewImage == true)
                        l_ImageSelectedList.Add(l_imageElement);
                }
            }

            return l_ImageSelectedList;
        }

        /// <summary>
        /// Retourne la liste des imagettes selectionn
        /// </summary>
        /// <returns></returns>
        public List<ImageElement> GetSelectedImagette()
        {
            List<ImageElement> l_ImageSelectedList = null;

            //Si il n'y a qu'un element alors je consdre qu'il est selectionn dans tous les cas
            if (principaleView.Children.Count == 1)
            {
               l_ImageSelectedList = new List<ImageElement>();
               l_ImageSelectedList.Add((principaleView.Children[0] as Border).Tag as ImageElement);
            }
            else
            { //Sinon on parcourt les elements
                l_ImageSelectedList = GetSelectElementOfChildren(listImageNonClasser.Children);
                l_ImageSelectedList.AddRange(GetSelectElementOfChildren(listImageASupprimer.Children));
                l_ImageSelectedList.AddRange(GetSelectElementOfChildren(listImageADeterminer.Children));
                l_ImageSelectedList.AddRange(GetSelectElementOfChildren(listImageAGarder.Children));
            }

            return l_ImageSelectedList;
        }

        /// <summary>
        /// Retourne la liste des images selectionn parmi les enfants
        /// </summary>
        /// <param name="in_children"></param>
        /// <returns></returns>
        public List<ImageElement> GetSelectElementOfChildren(UIElementCollection in_children)
        {
            List<ImageElement> l_ImageSelectedList = new List<ImageElement>();

            foreach (Border l_childBorder in in_children)
            {
                ImageElement l_imageElement = l_childBorder.Tag as ImageElement;
                if (l_imageElement.Select == true)
                    l_ImageSelectedList.Add(l_imageElement);
            }

            return l_ImageSelectedList;
        }

        void ViewImage_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //Sur double clique
            if (e.ClickCount == 2)
            {
                //On affiche l'image en grand
                if (popupView.Children.Count == 0)
                {
                    Image l_image = new Image();
                    Image l_selectImage = (sender as Image);
                    l_image.Source = l_selectImage.Source;
                    l_image.Width = ((principaleView.Parent) as Border).ActualWidth;
                    l_image.Height = ((principaleView.Parent) as Border).ActualHeight;
                    l_image.MouseLeftButtonDown += new MouseButtonEventHandler(PopupImage_MouseLeftButtonDown);

                    popupView.Children.Add(l_image);
                }
                else
                {
                    popupView.Children.Clear();
                }

            }
            //Sur simple clique
            if (e.ClickCount == 1)
            { 
                //Selection de l'image
                ImageElement l_selectImage = (sender as Image).Tag as ImageElement;
                if (l_selectImage.SelectViewImage == true)
                    l_selectImage.SelectViewImage = false;
                else 
                    l_selectImage.SelectViewImage = true;
            }
        }

       /// <summary>
       /// Sur double clic sur une images on affiche l'image en grand
       /// </summary>
       /// <param name="sender"></param>
       /// <param name="e"></param>
        void PopupImage_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //Sur double clique
            if (e.ClickCount == 2)
            {
                //On affiche l'image en grand
                if (popupView.Children.Count == 0)
                {
                    Image l_image = new Image();
                    Image l_selectImage = (sender as Image);
                    l_image.Source = l_selectImage.Source;
                    l_image.Width = principaleView.ActualWidth;
                    l_image.Height = principaleView.ActualHeight;
                    l_image.MouseLeftButtonDown += new MouseButtonEventHandler(ViewImage_MouseLeftButtonDown);

                    popupView.Children.Add(l_image);
                }
                else
                {
                    popupView.Children.Clear();
                }
            }
        }

        /// <summary>
        /// Desaffiche une image
        /// </summary>
        /// <param name="in_image"></param>
        public void Unshow()
        {
            //On met les images a null pour liberer de la memoire
            for (int l_counter = 1; l_counter < principaleView.Children.Count; l_counter++)
            {
                ((principaleView.Children[l_counter] as Border).Tag as ImageElement).Select = false;
                ((principaleView.Children[l_counter] as Border).Tag as ImageElement).ViewImage = null;
            }

            //On supprimer les enfants
            if(principaleView.Children.Count > 1)
                principaleView.Children.RemoveRange(1, principaleView.Children.Count - 1);

            //On supprime les colonnes et ligne en trop
            if(principaleView.ColumnDefinitions.Count > 1)
                principaleView.ColumnDefinitions.RemoveRange(1, principaleView.ColumnDefinitions.Count - 1);
            if(principaleView.RowDefinitions.Count > 1)
                principaleView.RowDefinitions.RemoveRange(1, principaleView.RowDefinitions.Count - 1);

            rowOrColunm = false;
        }

        /// <summary>
        /// Liberation des listes
        /// </summary>
        public void Clear()
        {
            ImageDirectory[] l_imageDirectory = new ImageDirectory[directoryList.Count];
            directoryList.CopyTo(l_imageDirectory);
            //On supprime les repertoires
            foreach (ImageDirectory l_directory in l_imageDirectory)
            {
                RemDirectoy(l_directory.Name);
            }

            //On recre un reperoire
            AddDirectory(A_GARDER);

            //On vide les listes
            listImageNonClasser.Children.Clear();
            listImageASupprimer.Children.Clear();
            listImageADeterminer.Children.Clear();
            listImageAGarder.Children.Clear();
            
            //On effece la vue principale
            principaleView.Children.Clear();
            if(principaleView.ColumnDefinitions.Count > 1)
                principaleView.ColumnDefinitions.RemoveRange(1, principaleView.ColumnDefinitions.Count - 1);
            if (principaleView.RowDefinitions.Count > 1)
                principaleView.RowDefinitions.RemoveRange(1, principaleView.RowDefinitions.Count - 1);

            rowOrColunm = false;
        }

        /// <summary>
        /// Selectionne une image
        /// </summary>
        /// <param name="in_image"></param>
        public void Select(ImageElement in_image)
        {
            if (in_image.Select)
                in_image.Select = false;
            else
                in_image.Select = true;

            Show(in_image);
        }

        /// <summary>
        /// Deselectionne toutes les images
        /// </summary>
        public void UnSelectAll()
        {
            List<ImageElement> l_imageList = GetSelectedImage();
            foreach (ImageElement l_image in l_imageList)
            {
                l_image.SelectViewImage = false;
            }
        }

        /// <summary>
        /// Chargement d'un repertoire
        /// </summary>
        public void AnalyseDirectory(string in_filePath)
        {
            //On vide les listes deja charg
            //ImageElementManager.Manager.Clear();

            //Evenement de debut de chargement
            if(LoadBegin != null)
                LoadBegin();

            //Recherche des images d'un reperoire
            List<string> l_imageList = new List<string>();
            foreach (string l_directory in listFileStringExtension)
            {
                string[] l_imageArray = Directory.GetFiles(in_filePath, l_directory);
                l_imageList.AddRange(l_imageArray);
            }

            int l_listLength = l_imageList.Count;
            int l_counter = 0;
            //Pout toutes les images
            foreach (string l_imagePath in l_imageList)
            {
                l_counter++;
                
                //Creation et chargement d'une image
                ImageElement l_currentImage = new ImageElement(l_imagePath);
                l_currentImage.Cursor = System.Windows.Input.Cursors.Hand;
                l_currentImage.ImageState = ImageState.NON_CLASSE;

                //Envoi de la progression
                if (LoadProgress != null)
                    LoadProgress(Convert.ToInt32((double)l_counter / (double)l_listLength * 100.0), l_imagePath);

                //Ajout de l'evement de selection de l'image
                l_currentImage.MouseUp += new MouseButtonEventHandler(ImageMouseUpCallBack);
                l_currentImage.MouseWheel += new MouseWheelEventHandler(ImageMouseWheelCallBack);
            }

            //Initialiser le repertoire de base
            directoryBase = in_filePath;

            imageAGarderDirectory.DirectoryBase = directoryBase;
            imageADeterminerDirectory.DirectoryBase = directoryBase;
            imageASupprimerDirectory.DirectoryBase = directoryBase;
            foreach (ImageDirectory l_imageDirectory in directoryList)
            {
                l_imageDirectory.DirectoryBase = directoryBase;
            }

            //Evenement de fin de chargement
            if (LoadEnd != null)
                LoadEnd();
        }

        /// <summary>
        /// Mouse Whell evenement
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ImageMouseWheelCallBack(object sender, MouseWheelEventArgs e)
        {
            ImageElement l_currentImage = (sender as ImageElement);
        }

        /// <summary>
        /// Declenche la copie des images dans les repertoires
        /// </summary>
        public void Transfert(bool in_applyRotation)
        {
            try
            {
                //Mise a jour du nombre d'image total a copier
                ComputeNbImageInDirectory();

                //On met le compteur de copie a 0
                _currentCopyProgress = 0;

                //Debut de copie
                if (LoadBegin != null)
                    LoadBegin();

                //Copie des repertoires
                imageADeterminerDirectory.CopieProgress += new ImageDirectory.CopieProgressEvent(CopieProgress);
                imageADeterminerDirectory.Transfert(in_applyRotation);
                imageADeterminerDirectory.CopieProgress -= new ImageDirectory.CopieProgressEvent(CopieProgress);

                imageASupprimerDirectory.CopieProgress += new ImageDirectory.CopieProgressEvent(CopieProgress);
                imageASupprimerDirectory.Transfert(in_applyRotation);
                imageASupprimerDirectory.CopieProgress += new ImageDirectory.CopieProgressEvent(CopieProgress);

                foreach (ImageDirectory l_imageDirectory in directoryList)
                {
                    l_imageDirectory.CopieProgress += new ImageDirectory.CopieProgressEvent(CopieProgress);
                    l_imageDirectory.Transfert(in_applyRotation);
                    l_imageDirectory.CopieProgress += new ImageDirectory.CopieProgressEvent(CopieProgress);
                }
            }
            catch (Exception ex)
            { }
            finally
            {
                //Fin de copie
                if (LoadEnd != null)
                    LoadEnd();
            }
        }

        /// <summary>
        /// Calcul le nombre total d'image dans les repertoires a copier
        /// </summary>
        private void ComputeNbImageInDirectory()
        {
            _nbImageInDirectories = imageADeterminerDirectory.NbImageInDirectory;
            _nbImageInDirectories += imageASupprimerDirectory.NbImageInDirectory;
            foreach (ImageDirectory l_imageDirectory in directoryList)
            {
                _nbImageInDirectories += l_imageDirectory.NbImageInDirectory;
            }
        }

        /// <summary>
        /// Evenement de copie de fichier
        /// </summary>
        /// <param name="in_text"></param>
        void CopieProgress(string in_text)
        {
            _currentCopyProgress++;

            if (LoadProgress != null)
            {
                int l_pourcentage = Convert.ToInt32( Math.Floor(((double)_currentCopyProgress / (double)_nbImageInDirectories) * 100.0));
                LoadProgress(l_pourcentage, in_text);
            }
        }
        #endregion

        #region callback
        /// <summary>
        /// Sur selection d'une image
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ImageMouseUpCallBack(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == 1 && e.ChangedButton == MouseButton.Left)
            {
                Select(sender as ImageElement);
            }
            else
            {
                if (e.ClickCount == 1 && e.ChangedButton == MouseButton.Right)
                {
                    ExifMetaData l_exifWindow = new ExifMetaData();

                    l_exifWindow.ImageExifMetadata = (sender as ImageElement).ImageExifMetadata;

                    l_exifWindow.Owner = App.Current.MainWindow;

                    l_exifWindow.Show();
                }
            }
        }
        #endregion
    }
}
